// Low-level handling of asynchronous messages.
//
// $Id: async.cc,v 1.10 1994/08/08 15:22:53 inge Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <assert.h>
#include <stdlib.h>
#include "async.h"
#include "prot-a-LyStr.h"
#include "connection.h"

Async_storage::Async_storage(Connection *c)
  : port(c), is_registered(FALSE), pending_async()
{
    for (int i=0; i < MAX_ASYNC; i++)
	admin_table[i] = NULL;
}

Status 
Async_storage::handle_async(int this_many)
{
    while (this_many-- > 0 && !pending_async.empty())
    {
	Pending next = pending_async.remove_front();
	admin_table[next.type]->handle_message(next.object);
    }

    return pending_async.empty() ? st_ok : st_pending;
}

Connection *
Async_storage::connection() const
{
    return port;
}

void 
Async_storage::register_administrator(Async_name n, 
				      Async_administrator* adm)
{
    // FIXME++. Are enums really always unsigned?  Check with Bjarne.
    // assert(0 <= n);
    assert(n < MAX_ASYNC);
    assert(admin_table[n] == NULL);
    admin_table[n] = adm;
    if (!is_registered)
    {
	port->register_async((void*)this, Async_storage::parse_and_store);
	is_registered = TRUE;
    }
}

void 
Async_storage::parse_and_store(void *async_storage,
			       prot_a_LyStr async_msg)
{
    Async_storage *ast = (Async_storage *)async_storage;

    async_msg.stol();			// Skip 'number of items' field.
    long type = async_msg.stol();

    if (type >= 0 && type < MAX_ASYNC && ast->admin_table[type] != NULL)
    {
	Pending foo;   
	foo.object = ast->admin_table[type]->parse(async_msg);
	foo.type = Async_name(type);
	ast->pending_async.append(foo);
    }
}


// ================================================================
//                  Async_administrator methods


Async_administrator::Async_administrator(Async_storage *ast)
  : handlers(NULL), storage(ast), is_registered(FALSE)
{
}

Async_administrator::~Async_administrator()
{
    // FIXME: Why should we not delete the list?  There used to be an
    // assert here:
    // assert(handlers==NULL);
    while (handlers != NULL) {
	Hinfo *next = handlers->next;
	delete handlers;
	handlers = next;
    }
}

void * 
Async_administrator::register_handler(void* userdata, 
				      void* handler,
				      Async_name name)
{
    handlers = new Hinfo(handler, userdata, handlers);
    if (!is_registered)
    {
	storage->register_administrator(name, this);
	is_registered = TRUE;
    }

    return handlers;
}


void 
Async_administrator::unregister_handler(void* unwanted)
{
    for (Hinfo **ptr = &handlers; *ptr != NULL; ptr = &(*ptr)->next)
    {
	if (*ptr == unwanted)
	{
	    Hinfo *del = *ptr;
	    *ptr = (*ptr)->next;
	    delete del;
	    return;
	}
    }
    
    abort();
}


Async_administrator::Hinfo::Hinfo(void *hhandler,
				  void *uuserdata, 
				  Hinfo *nnext)
  : userdata(uuserdata), handle(hhandler), next(nnext)
{
}

Async_administrator::Hinfo *
Async_administrator::handler_list_head() const
{
    return handlers;
}
