// Low-level handling of asynchronous messages.
//
// $Id: async.h,v 1.7 1994/08/05 22:00:31 inge Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef ASYNC_
#define ASYNC_

#include <SLList.h>
#include "misc.h"

const MAX_ASYNC=255;

class prot_a_LyStr;
class Connection;
class Async_administrator;
class Async_message;

// This description is up-to-date with LysKOM server 1.3.3.
enum Async_name {
    ASYNC_NEW_TEXT,		// 0
    ASYNC_LOGOUT_OBSOLETE,	// 1: OBSOLETE: don\'t use
    ASYNC_LOGIN_OBSOLETE,	// 2: OBSOLETE: don\'t use
    ASYNC_CONF_DELETED,		// 3
    ASYNC_CONF_CREATED,		// 4
    ASYNC_CONF_CHANGE_NAME,	// 5
    ASYNC_I_AM_ON,		// 6: Sent when a person logs in,
                                //    changes conference, or sends a 
                                //    "Change what I am doing".
    ASYNC_SYNC_DB,		// 7
    ASYNC_FORCED_LEAVE_CONF,	// 8
    ASYNC_LOGIN,		// 9
    ASYNC_BROADCAST_OBSOLETE,	// 10: OBSOLETE: don\'t use
    ASYNC_LYSKOM_IS_FULL,	// 11
    ASYNC_TEXT_MESSAGE,		// 12
    ASYNC_LOGOUT,		// 13
};

class Async_storage {
  public:
    // Methods intended for use by the end user.
    Async_storage(Connection *);

    Status        handle_async(int this_many=1);
    Connection  * connection() const;

    // Methods intended for use by Async_administrator and its derived classes.
    void          register_administrator(Async_name, 
					 Async_administrator*);

    // Function intended to be called from class Connection (via an
    // indirection function).
    static void   parse_and_store(void *async_storage, 
				  prot_a_LyStr async_msg);

  private:
    Connection           * port;
    Bool                   is_registered; 
    Async_administrator  * admin_table[MAX_ASYNC];
    class Pending {
      public:
	Async_name             type;
	const Async_message  * object;
    };
    SLList<Async_storage::Pending>  pending_async;
};

class Async_administrator {
  public:
    Async_administrator(Async_storage *);
    virtual const Async_message* parse(prot_a_LyStr &) const =0;
    virtual void handle_message(const Async_message *) =0;
    virtual ~Async_administrator();
  protected:
    void* register_handler(void* userdata, void* handler, Async_name);
    void unregister_handler(void*);

    struct Hinfo {
	Hinfo(void *handler, void *userdata, Hinfo *next);
	void *userdata;
	void *handle;
	Hinfo *next;
    };
    Hinfo *handler_list_head() const;
  private:
    Hinfo *handlers;
    Async_storage *storage;
    Bool is_registered;
    Async_administrator& operator=(const Async_administrator&); // Non-existing
//    Async_administrator(const Async_administrator&);		// Non-existing
};

class Async_message {
    // This is a very small class.
};
    

#endif
