// Higher-level handling of asynchronous messages.
//
// $Id: async_dispatcher.cc,v 1.4 1994/08/10 07:18:31 inge Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include <stdlib.h>
#include "connection.h"
#include "async_dispatcher.h"
#include "async_text_created.h"
#include "async_text_message.h"


Async_dispatcher::Async_dispatcher(Connection *conn)
    : port(conn),
    text_created_admin(NULL), text_message_admin(NULL)
{
    storage = new Async_storage(port);
}


Async_dispatcher::~Async_dispatcher()
{
    delete storage;
    delete text_created_admin;
    delete text_message_admin;
}


// ================================================================


//Async_dispatcher::Async_dispatcher &operator=(const Async_dispatcher &)
//{
//}

  
void *
Async_dispatcher::register_text_created_handler(void *userdata,
					 	Text_created_handlerp hnd)
{
    if (text_created_admin == NULL) {
	text_created_admin = new Async_text_created_administrator(storage);
	storage->register_administrator(ASYNC_NEW_TEXT, text_created_admin);
    }

    return text_created_admin ->register_handler(userdata, hnd);
}


void *
Async_dispatcher::register_text_message_handler(void *userdata,
						Text_message_handlerp hnd)
{
    if (text_message_admin == NULL) {
	text_message_admin = new Async_text_message_administrator(storage);
	storage->register_administrator(ASYNC_TEXT_MESSAGE, 
					text_message_admin);
    }

    return text_message_admin->register_handler(userdata, hnd);
}


void
Async_dispatcher::unregister_text_created_handler(void *unwanted)
{
    text_created_admin->unregister_handler(unwanted);
}


void
Async_dispatcher::unregister_text_message_handler(void *unwanted)
{
    text_message_admin->unregister_handler(unwanted);
}


Status
Async_dispatcher::handle_async(int this_many)
{
    storage->handle_async(this_many);
}
