// Objects for handling the asynchronous message "Text message"
//
// $Id: async_text_message.cc,v 1.1 1994/08/05 21:58:14 inge Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <assert.h>
#include <stdlib.h>
#include "async_text_message.h"
#include "prot-a-LyStr.h"
#include "connection.h"


Async_text_message::Async_text_message()
    : recipient_m(0), sender_m(0), message_m()
{
}

Async_text_message::Async_text_message(const Async_text_message &t)
    : recipient_m(t.recipient_m), sender_m(t.sender_m), message_m(t.message_m)
{
}

Async_text_message::~Async_text_message()
{
}

Async_text_message &
Async_text_message::operator =(const Async_text_message &t)
{
    recipient_m = t.recipient_m;
    sender_m = t.sender_m;
    message_m = t.message_m;

    return *this;
}

Pers_no
Async_text_message::recipient() const
{
    return recipient_m;
}

Pers_no
Async_text_message::sender() const
{
    return sender_m;
}

const LyStr &
Async_text_message::message() const
{
    return message_m;
}


Async_text_message::Async_text_message(prot_a_LyStr &p)
{
    recipient_m = p.stol();
    sender_m = p.stol();
    message_m = p;
}


// ================================================================
//         Async_text_message_administrator methods


Async_text_message_administrator::
Async_text_message_administrator(Async_storage *as)
  : Async_administrator(as)
{
}

Async_text_message_administrator::~Async_text_message_administrator()
{
}

void *
Async_text_message_administrator::register_handler(void *userdata, 
						   Handlerp hnd)
{
    return Async_administrator::register_handler(userdata, (void*)hnd, 
						 ASYNC_TEXT_MESSAGE);
}


void
Async_text_message_administrator::unregister_handler(void* hnd)
{
    Async_administrator::unregister_handler(hnd);
}


const Async_message *
Async_text_message_administrator::parse(prot_a_LyStr &msg) const
{
    return new Async_text_message(msg);
}


void 
Async_text_message_administrator::handle_message(const Async_message *msg)
{
    for (struct Hinfo* ptr = handler_list_head();
	 ptr != NULL;
	 ptr = ptr->next)
    {
	Async_text_message *m = (Async_text_message*)msg;
	(*(Handlerp)ptr->handle)(ptr->userdata, *m);
    }
}
