// Read texts in comment-order.
//
// $Id: comment-order.cc,v 1.2 1994/07/16 17:43:06 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <assert.h>
#include <iostream.h>
#include <stdlib.h>

#include "conference.h"
#include "connection.h"
#include "get_membership_q.h"
#include "kom-types.h"
#include "maps.h"
#include "membership.h"
#include "misc.h"
#include "comment-order.h"
#include "text-mapping.h"
#include "text.h"

// ================================================================ 
//                    Constructors and Destructors                  


Comment_order::Comment_order(Session *sess, Conf_no cno, Pers_no Commenter)
    : session(sess), persons(sess->persons()), 
      conferences(sess->conferences()), map(sess->maps()[cno]), 
      texts(sess->texts()),
      conf_no(cno), pers_no(Commenter), pending_membership_question(NULL),
      membership(NULL),
      unread_texts(), first_unchecked(0)
{
}
 

//Comment_order::Comment_order(const Comment_order &ro)
//    : conferences(ro.conferences), map(ro.map),
//      conf_no(ro.conf_no), pers_no(ro.pers_no),
//      unread_texts(ro.unread_texts), first_unchecked(ro.first_unchecked)
//{
//}


Comment_order::~Comment_order()
{
    delete pending_membership_question;
    delete membership;
}


// ================================================================ 
//

//const Comment_order & 
//Comment_order::operator = (const Comment_order &)
//{
//    cerr << "Operator= for class Comment_order is not yet implemented\n";
//    return *this;
//}


// ================================================================


int
Comment_order::prefetch_map(int texts)
{
    Conference conf = conferences[conf_no];
    if (membership == NULL && pending_membership_question == NULL)
    {
	pending_membership_question = 
	    new get_membership_question(conferences.connection(), 
					pers_no, conf_no);
    }

    switch (conf.prefetch())
    {
    case st_pending:
	// We can't do anything sensible unless we have the confstat.
	return 1 + (pending_membership_question != NULL
		    && pending_membership_question->status() == st_pending);

    case st_error:
	// Now what? +++FIXME///
	// (Note: This case CAN occur if the conference is being
	// deleted during a session).
	return 0;

    case st_ok:
        {
	    Local_text_no first_local = conf.first_local_no();
	    Local_text_no limit = first_local + conf.no_of_texts();

	    if (unread_texts.length() < texts && first_unchecked < limit)
	    {
		if (pending_membership_question != NULL)
		{
		    switch (pending_membership_question->status())
		    {
		    case st_pending:
			return 1;
			
		    case st_error:
			// Now what? +++FIXME///
			return 0;
		    
		    case st_ok:
			receive_membership();
			break;
		    
		    default:
			abort();
		    }
		}

		if (first_unchecked < first_local)
		    first_unchecked = first_local;

		while (first_unchecked < limit 
		       && unread_texts.length() < texts)
		{
		    if (!membership->is_Comment(first_unchecked))
		    {
			// FIXME+++: The "50" on the next line used to be
			// "texts", but that caused a very large question
			// to be issued. The map question should be split
			// into severaly small chunks, but I'm not sure
			// where the splitting should be done.
		        // Temporarily, do it this way.
			
			if (map.prefetch(first_unchecked, 50) == st_pending)
			    return 1;
			
			Text_no tno = map[first_unchecked];
			if (tno != 0)
			    unread_texts.append(tno);
		    }
		    
		    first_unchecked++;
		}
	    }
	}
	break;

    default:
	abort();
    }

    return 0;
}

int
Comment_order::prefetch(int required_texts,
			int max_question,
			int min_question)
{
    Conference conf = conferences[conf_no];
    if (membership == NULL && pending_membership_question == NULL)
    {
	pending_membership_question = 
	    new get_membership_question(conferences.connection(), 
					pers_no, conf_no);
    }

    switch (conf.prefetch())
    {
    case st_pending:
	// We can't do anything sensible unless we have the confstat.
	return 1 + (pending_membership_question != NULL
		    && pending_membership_question->status() == st_pending);

    case st_error:
	// Now what? +++FIXME///
	// (Note: This case CAN occur if the conference is being
	// deleted during a session).
	return 0;

    case st_ok:
	break;

    default:
	abort();
    }

    if (pending_membership_question != NULL)
    {
	switch (pending_membership_question->status())
	{
	case st_pending:
	    return 1;
			
	case st_error:
	    // Now what? +++FIXME///
	    return 0;
	    
	case st_ok:
	    receive_membership();
	    break;
	    
	default:
	    abort();
	}
    }
    
    Local_text_no first_local = conf.first_local_no();
    Local_text_no limit = first_local + conf.no_of_texts();
    
    if (unread_texts.length() < texts && first_unchecked < limit)
    {
	if (first_unchecked < first_local)
	    first_unchecked = first_local;
	
	while (first_unchecked < limit 
	       && unread_texts.length() < texts)
	{
	    if (!membership->is_read(first_unchecked))
	    {
		// FIXME+++: The "50" on the next line used to be
		// "texts", but that caused a very large question
		// to be issued. The map question should be split
		// into severaly small chunks, but I'm not sure
		// where the splitting should be done.
		        // Temporarily, do it this way.
			
			if (map.prefetch(first_unchecked, 50) == st_pending)
			    return 1;
			
			Text_no tno = map[first_unchecked];
			if (tno != 0)
			    unread_texts.append(tno);
		    }
		    
		    first_unchecked++;
		}
	    }
	}

    for (Pix ix = unread_texts.first();
	 ix != 0 && ((required_texts > 0 && questions < max_question)
		     || questions < min_question);
	 unread_texts.next(ix))
    {
	Text t = texts[unread_texts(ix)];
	switch (t.prefetch())
	{
	case st_pending:
	    questions++;
	    break;
	    
	case st_error:
	    // This is strange. The text must have been deleted very
	    // resently. Remove it. (This may cause some problems if
	    // this is the text we are about to show).
	    unread_texts.del(ix, -1);
	    break;

	case st_ok:
	    // Go ahead and prefetch some other stuff...
	    {
		int nq = t.prefetch_for_display(required_texts > 0 ? 
						max_question - questions : 
						min_question - questions,
						session);
		questions += nq;
		if (nq == 0)
		    required_texts--;
	    }
	    break;

	default:
	    abort();
	}
    }

    return questions;
}

			
Prompt
Comment_order::next_prompt()
{
    while (prefetch_map(1) > 0)
    {	
	switch (Connection::drain(100))
	{
	case st_pending:
	    return TIMEOUT_PROMPT;
	case st_error:
	    return PROMPT_ERROR;
	case st_ok:
	    break;
	}
    }

    if (!pending_footnote.empty())
	return NEXT_FOOTNOTE;
    else if (!pending_comments.empty())
	return NEXT_COMMENT;
    else if (unread_texts.empty())
	return NO_MORE_TEXT;
    else
	return NEXT_TEXT;
}


//
// Return the next text to be Comment within this conference.
// Return 0 if there is no more unread text.
//

Text_no
Comment_order::next_text(Bool remove)
{
    Text_no tno;
    
    while (prefetch_map(1) > 0)
	if (Connection::drain(100) == st_error)
	    return 0;


    if (remove)
	if (!pending_footnotes.empty())
	    tno = pending_footnotes.pop();
	else if (!pending_comments.empty())
	    tno = pending_comments.pop();
	else if (!unread_texts.empty())
	    tno = unread_texts.remove_front();
        else
	    tno = 0;
    else
	if (!pending_footnotes.empty())
	    tno = pending_footnotes.top();
	else if (!pending_comments.empty())
	    tno = pending_comments.top();
	else if (!unread_texts.empty())
	    tno = unread_texts.front();
        else
	    tno = 0;

    if (remove && tno != 0)
    {
	// Add comments/footnotes
	Text t = texts[tno];
	Local_text_no loc;
	for (int i = t.num_footnote_in() -1; i >= 0; i--)
	{
	    loc = texts[t.footnote_in(i)].local_no_recipient(conf_no);
	    if (loc)
		pending_footnotes.push(loc);
	}
	for (i = t.num_comment_in() -1; i >= 0; i--)
	{
	    loc = texts[t.comment_in(i)].local_no_recipient(conf_no);
	    if (loc)
		pending_comments.push(loc);
	}
    }
    return tno;
}


Bool
Comment_order::empty()
{
    while (prefetch_map(1) > 0
	   && Connection::drain(100) != st_error)
	continue;

    return (Bool) unread_texts.empty();
}


long
Comment_order::min_num_unread()
{
    return unread_texts.length();
}

 
void
Comment_order::receive_membership()
{
    assert (membership == NULL);
    membership = new Membership;
    *membership = pending_membership_question->result();
    delete pending_membership_question;
    pending_membership_question = NULL;
}    

long
Comment_order::max_num_unread()
{
    Conference conf = conferences[conf_no];
    if (!conf.present())
	return LONG_MAX;

    if (pending_membership_question != NULL
	&& pending_membership_question->status() == st_ok)
    {
	receive_membership();
    }

    if (membership == NULL)
    {
	// +++ We could subtract any texts here that we know are
	// deleted. Such information may exist in the text-mapping.
	return conf.no_of_texts();
    }

    long result = unread_texts.length();

    Local_text_no limit = conf.first_local_no() + conf.no_of_texts();

#if 0
    // This is SLOW when you have several thousand unread in I]M.
    for (Local_text_no ln = ((first_unchecked > membership->last_text_Comment())
			     ? first_unchecked : membership->last_text_Comment());
	 ln < limit; 
	 ln++)
    {
	if (!membership->is_Comment(ln))
	{
	    // +++ Should check if we know that this text is deleted.
	    result++;
	}
    }
#else
    if (first_unchecked > membership->last_text_Comment())
	result += limit - first_unchecked;
    else
	result += (limit - membership->last_text_Comment()
		   - membership->num_Comment_after_last_text_Comment());
#endif

    return result;    
}

Conf_no
Comment_order::confno() const
{
    return conf_no;
}
