// Cache of conf-statuses.
//
// $Id: conf-cache.cc,v 1.12 1994/08/06 20:26:54 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <stdlib.h>
#include "conf-cache.h"
#include "question.h"
#include "misc.h"
#include "error-codes.h"

static Conf_stat bad_conf_stat;
static Conf_stat zero_conf_stat(KOM_CONF_ZERO);

Conf_cache::Conf_cache(Connection *c)
{
    conn = c;
}

//Conf_cache::Conf_cache(const Conf_cache& c)
//{
//    *this = c;
//}

//Conf_cache &
//Conf_cache::operator=(const Conf_cache& c)
//{
//    cerr << "Conf_cache::operator=(const Conf_cache& c) unimp\n";
//    abort();
//}

Conf_cache::~Conf_cache()
{
}

Connection *
Conf_cache::connection() const
{
    return conn;
}

Bool
Conf_cache::present(Conf_no cno) const
{
    return statusmap.present(cno);
}

//void
//Conf_cache::clear()
//{
//    cerr << "Unimp\n";
//    abort();
//}


Status
Conf_cache::prefetch(Conf_no cno)
{
    if (cno <= 0)
	return st_error;

    if (statusmap.present(cno))
    {
	if (statusmap[cno].valid())
	    return st_ok;
	else
	    return st_error;
    }
    else
    {
	if (!pendingmap.present(cno))
	{
	    pendingmap[cno] = new get_conf_stat_question(conn, cno);
	    return st_pending;
	}
	else
	{
	    switch (pendingmap[cno]->status())
	    {
	    case st_pending:
		return st_pending;
	    case st_ok:
		statusmap[cno] = pendingmap[cno]->result();
		pendingmap.delete_entry(cno);
		return st_ok;
	    case st_error:
		statusmap[cno] = Conf_stat(pendingmap[cno]->error());
		pendingmap.delete_entry(cno);
		return st_error;
	    default:
		abort();
	    }
	}
    }
    abort();
}


Conf_stat
Conf_cache::get(Conf_no cno)
{
    if (cno <= 0)
	return zero_conf_stat;

    if (statusmap.present(cno))
    {
	Conf_stat FIXME = statusmap[cno];
	return FIXME;
    }
    else if (pendingmap.present(cno))
    {
	get_conf_stat_question *gtr = pendingmap[cno];
	if (gtr->receive() == st_ok)
	    statusmap[cno] = gtr->result();
	else
	    statusmap[cno] = Conf_stat(gtr->error());
	pendingmap.delete_entry(cno);
	Conf_stat FIXME=statusmap[cno];
	return FIXME;
    }
    else		
    {
	// Not present in cache, and no outstanding question.
	get_conf_stat_question gt(conn, cno);
	if (gt.receive() == st_ok)
	    statusmap[cno] = gt.result();
	else
	{
	    // statusmap[cno] = Conf_stat(gt.error());
	    Conf_stat FIXME(gt.error());
	    statusmap[cno] = FIXME;
	}
	
	return statusmap[cno];
    }
    abort();
}

//void
//Conf_cache::remove(Conf_no cno)
//{
//    abort();
//}

Bool
Conf_cache::Ccmap::present(Conf_no c) const
{
    // contains() really should be a const member function. This is a 
    // workaround. The line below should be substituted.
    return ((Conf_cache::Ccmap*)this)->contains(c) ? TRUE : FALSE;
    // return contains(c) ? TRUE : FALSE;
}

Conf_stat &
Conf_cache::Ccmap::operator[](Conf_no c)
{
    return Conf_noConf_statCHMap::operator[](c);
}


Conf_cache::Ccmap::Ccmap()
    : Conf_noConf_statCHMap(bad_conf_stat, 50)
{
}

Bool
Conf_cache::Ccreqmap::present(Conf_no c)
{
    return contains(c) ? TRUE : FALSE;
}

void
Conf_cache::Ccreqmap::delete_entry(Conf_no c)
{
    delete operator[](c);
    del(c);
}

get_conf_stat_question *&
Conf_cache::Ccreqmap::operator[](Conf_no c)
{
    return Conf_noget_conf_stat_question_ptrCHMap::operator[](c);
}

static get_conf_stat_question_ptr bad_ptr = NULL;

Conf_cache::Ccreqmap::Ccreqmap()
    : Conf_noget_conf_stat_question_ptrCHMap(bad_ptr)
{
}

Conf_cache::Ccreqmap::~Ccreqmap()
{
    for (Pix i = first(); i != 0; next(i))
    {
	delete contents(i);
    }
}
