// Fetch-on-demand conference object.
//
// $Id: conference.h,v 1.12 1994/07/16 17:43:42 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef CONFERENCE__
#define CONFERENCE__

#include "misc.h"
#include "LyStr.h"
#include "kom-types.h"

class Conf_cache;
class Conf_type;
class Connection;

class Conference {
  public:
    Conference(Conf_no, Conf_cache *);
    Conference(const Conference &);
    ~Conference();

    const Conference & operator = (const Conference &);

    Status      prefetch()   const;
    Bool        valid()      const;
    int		errno()	     const;
    Bool	present()    const;
    Conf_no     conf_no()    const;
    Connection* connection() const;

    Pers_no		creator()		const;
    struct tm		creation_time()		const;
    Text_no		presentation()		const;
    Conf_no		supervisor()		const;
    Conf_no		permitted_submitters()	const;
    Conf_no		super_conf()		const;
    Conf_type		type()			const;
    struct tm		last_written()		const;
    Text_no		msg_of_day()		const;
    Garb_nice		nice()			const;
    LyStr		name()			const;
    unsigned short	no_of_members()		const;
    Local_text_no	first_local_no()	const;
    unsigned long	no_of_texts()		const;

  private:
    Conf_no      cno;
    Conf_cache * cache;
};



class Conferences {
  public:
    Conferences();
    Conferences(Connection *);
    Conferences(const Conferences &);
    ~Conferences();

    const Conferences & operator = (const Conferences &);
    void  set_connection(Connection *);
    Connection *connection() const;

    Conference  operator[](Conf_no) const;

    Conf_cache * read_conf_cache() const;

  private:
    Connection  * conn;
    Conf_cache  * conf_cache;
};


#endif  // CONFERENCE__
