// Privilege bits.
//
// $Id: priv-bits.h,v 1.3 1994/07/16 17:45:41 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef PRIV_BITS__
#define PRIV_BITS__

#include "misc.h"

class LyStr;

/*
 * The privilige bits:  says what priviliges a person has.  E g
 *  if he is allowed to read texts he normally shouldn't be allowed
 *  to read. See file doc/security-levels.txt
 */
class Priv_bits {
  public:
    Priv_bits();
    Priv_bits(const Priv_bits&);
    const Priv_bits &operator =(const Priv_bits &);

    friend Priv_bits parse_priv_bits(LyStr &prot_a_str);

    Bool wheel() const;
    Bool admin() const;
    Bool statistic() const;
    Bool create_pers() const;
    Bool create_conf() const;
    Bool change_name() const;
    Bool extern_gw() const;
  private:
    unsigned int	pb_wheel	: 1;
    unsigned int	pb_admin	: 1;
    unsigned int	pb_statistic : 1;
    unsigned int	pb_create_pers: 1;
    unsigned int	pb_create_conf: 1;
    unsigned int	pb_change_name: 1;
    unsigned int	pb_extern_gw : 1;
    unsigned int	flg8	: 1; /* For future use. */
    unsigned int	flg9    : 1;
    unsigned int	flg10   : 1;
    unsigned int	flg11   : 1;
    unsigned int	flg12	: 1;
    unsigned int	flg13	: 1;
    unsigned int	flg14	: 1;
    unsigned int	flg15	: 1;
    unsigned int	flg16	: 1;
};

Priv_bits parse_priv_bits(LyStr &prot_a_str);

#endif
