// Handle reading of texts in linear order.
//
// $Id: read-order.h,v 1.12 1994/07/30 22:11:23 inge Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef READ_ORDER__
#define READ_ORDER__

#include <DLList.h>

#include "misc.h"
#include "kom-types.h"
#include "text-mapping.h"
#include "membership.h"

#include "session.h"

class Conferences;
class Persons;
class Maps;
class Texts;
class get_membership_question;

// An enum Prompt is returned by Read_order::prompt() and
// Conf_list::prompt().  Read_order::prompt() never returns NEXT_CONF.

enum Prompt {
    NEXT_CONF=1,	// Never returned by Read_order::prompt().
    NEXT_TEXT,
    NEXT_COMMENT, 
    NEXT_FOOTNOTE,
    NO_MORE_TEXT,
    TIMEOUT_PROMPT,		// It took too long time to find out
				// the next prompt.
    PROMPT_ERROR		// Can not find out the next prompt;
				// the connection might be lost.
};


// 
// This object determines what order the texts should be read
// within a conference.
//

class Read_order {
  public:
    Read_order(Session *sess, Conf_no cno, Pers_no reader);
    ~Read_order();

    // Know about this many texts. This only prefetches the map.
    // Returns number of pending questions, or -1 if an error occurs.
    int       prefetch_map(int texts);

    // Prefetch everything that is normally needed to display the
    // TEXTS first texts. "What is normally needed" is defined as
    // "what is needed by nilkom". Issue at most max_question
    // simultaneous questions. Continue to prefetch everything that
    // can reasonably be prefetched with at least min_question
    // questions thereafter. Return number of pending questions, or -1
    // if an error occurs.
    int	      prefetch(int texts = 5,
		       int max_question = 3,
		       int min_question = 1);

    Prompt    next_prompt();
    Text_no   next_text(Bool remove=FALSE);

    // Return TRUE if the conf is empty after the removal.
    Bool      remove_text(Text_no);
    Bool      empty();
    long      min_num_unread();
    long      max_num_unread();
    Conf_no   confno() const;

  private:
    Read_order(const Read_order &); // NYI
    const Read_order & operator = (const Read_order &);	// NYI

    Session      * session;
    Persons	 & persons;
    Conferences  & conferences;
    Texts	 & texts;
    Text_mapping   map;
    Conf_no        conf_no;
    Pers_no        pers_no;

    get_membership_question *pending_membership_question;
    Membership     membership;
    void           receive_membership();

    Local_text_no  first_unchecked;
    DLList<Text_no>  unread_texts;
};
#endif  // READ_ORDER__
