// Highlevel handling of a session to a LysKOM server.
//
// $Id: session.h,v 1.18 1994/08/10 07:20:16 inge Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef Session__
#define Session__

#include <stdlib.h>
#include <iostream.h>

#include "array.h"
#include "LyStr.h"
#include "kom-types.h"
#include "who-info.h"
#include "micro-conf.h"
#include "connection.h"
#include "async_dispatcher.h"
#include "mark.h"

#include "obsolete.h"		// For who_is_on_question.

#include "conference.h"
#include "person.h"
#include "maps.h"
#include "text.h"
//#include "memberships.h"   // +++NYI


// +++FIXME: Hur skall felhanteringen g} till?  Vi vill kunna f} ut
//           en felkod om n}got g}r fel.

class Session {
  public:
    Session(Connection *);
    ~Session();

    friend ostream &operator <<(ostream &, const Session &);

    Conferences &  conferences();
    Persons     &  persons();
    Maps        &  maps();
    Texts       &  texts();
    Pers_no        my_login() const;
    Session_no     my_session() const;

    // Create a new person.  If RESULT is non-NULL, the new person number will
    // be stored there.  An automatic login is performed if you are not already
    // logged in.  If AUTOLOGIN is non-NULL, it will be set if a auto-login 
    // was made.
    Status              create_person(const LyStr &name, const LyStr &password,
				      Pers_no *result = NULL, 
				      Bool *autologin = NULL);

    Status              login(Pers_no, const LyStr &, Bool invisible = FALSE);
    Status              add_member(Conf_no cno, Pers_no reader,
				   unsigned char priority, 
				   unsigned short placement);
    Status              send_message(Pers_no recipient, const LyStr &message);
    struct tm           get_time(); 
    Status              change_what_i_am_doing(const LyStr &str);
    Array<Who_info>     who_is_on();
    Array<Micro_conf>   lookup_name(const LyStr &pattern);
    Array<Pers_no>      regexp_lookup_persons(const LyStr &pattern);
    Array<Mark>         get_marks();

    Async_dispatcher  & async_dispatcher();
    Status              handle_async(int this_many = 1);

  private:
    Connection  * conn;
    Pers_no       logged_in_person;
    Session_no    session_number; // 0 if not yet known

    Conferences * all_confs;
    Persons     * all_persons;
    Maps        * all_maps;
    Texts       * all_texts;

    Async_dispatcher  * m_async_dispatcher;

    // Not implemented, since it doesn't make sense to copy sessions.
    Session(const Session &);
    const Session & operator = (const Session &);
};


#endif  // Session__
