// Fetch-on-demand cache for texts.
//
// $Id: text-cache.cc,v 1.8 1994/08/06 20:32:57 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <stdlib.h>
#include "error-codes.h"
#include "text-cache.h"
#include "question.h"
#include "misc.h"

static Text_stat bad_text_stat;
static Text_stat zero_text_stat(KOM_TEXT_ZERO);

Text_stat_cache::Text_stat_cache(Connection *c)
{
    conn = c;
}

Text_stat_cache::Text_stat_cache(const Text_stat_cache& c)
{
    *this = c;
}

Text_stat_cache &
Text_stat_cache::operator=(const Text_stat_cache& c)
{
    abort();
    return *this;
}

Text_stat_cache::~Text_stat_cache()
{
}

Bool
Text_stat_cache::present(Text_no cno) const
{
    return statusmap.present(cno);
}

void
Text_stat_cache::clear()
{
    cerr << "Unimp\n";
    abort();
}

Connection *
Text_stat_cache::connection() const
{
    return conn;
}


Status
Text_stat_cache::prefetch(Text_no cno)
{
    if (cno <= 0)
	return st_error;

    if (statusmap.present(cno))
    {
	if (statusmap[cno].valid())
	    return st_ok;
	else
	    return st_error;
    }
    else
    {
	if (!pendingmap.present(cno))
	{
	    pendingmap[cno] = new get_text_stat_question(conn, cno);
	    return st_pending;
	}
	else
	{
	    switch (pendingmap[cno]->status())
	    {
	    case st_pending:
		return st_pending;
	    case st_ok:
		statusmap[cno] = pendingmap[cno]->result();
		pendingmap.delete_entry(cno);
		return st_ok;
	    case st_error:
		statusmap[cno] = Text_stat(pendingmap[cno]->error());
		pendingmap.delete_entry(cno);
		return st_error;
	    default:
		abort();
	    }
	}
    }
    abort();
}


Text_stat
Text_stat_cache::get(Text_no cno)
{
    if (cno <= 0)
	return zero_text_stat;

    if (statusmap.present(cno))
    {
	// return statusmap[cno];
	Text_stat GCCBUG(statusmap[cno]);
	return GCCBUG;
    }
    else if (pendingmap.present(cno))
    {
	get_text_stat_question *gtr = pendingmap[cno];
	if (gtr->receive() == st_ok)
	    statusmap[cno] = gtr->result();
	else
	    statusmap[cno] = Text_stat(gtr->error());
	pendingmap.delete_entry(cno);
	Text_stat GCCBUG = statusmap[cno];
	return GCCBUG;
    }
    else		
    {
	// Not present in cache, and no outstanding question.
	get_text_stat_question gt(conn, cno);
	if (gt.receive() == st_ok)
	    statusmap[cno] = gt.result();
	else
	{
	    Text_stat GCCBUG(gt.error());
	    statusmap[cno] = GCCBUG;
	    // statusmap[cno] = Text_stat(gt.error());
	}
	// return statusmap[cno];
	Text_stat GCCBUG(statusmap[cno]);
	return GCCBUG;
    }
    abort();
}

//void
//Text_stat_cache::remove(Text_no cno)
//{
//    abort();
//}

Bool
Text_stat_cache::Tstmap::present(Text_no c) const
{
    // contains() really should be a const member function. This is a 
    // workaround. The line below should be substituted.
    return ((Text_stat_cache::Tstmap*)this)->contains(c) ? TRUE : FALSE;
    // return contains(c) ? TRUE : FALSE;
}

Text_stat &
Text_stat_cache::Tstmap::operator[](Text_no c)
{
    return Text_noText_statCHMap::operator[](c);
}


Text_stat_cache::Tstmap::Tstmap()
    : Text_noText_statCHMap(bad_text_stat, 50)
{
}

Bool
Text_stat_cache::Tstreqmap::present(Text_no c)
{
    return contains(c) ? TRUE : FALSE;
}

void
Text_stat_cache::Tstreqmap::delete_entry(Text_no c)
{
    delete operator[](c);
    del(c);
}

get_text_stat_question *&
Text_stat_cache::Tstreqmap::operator[](Text_no c)
{
    return Text_noget_text_stat_question_ptrCHMap::operator[](c);
}

static get_text_stat_question_ptr bad_ptr = NULL;

Text_stat_cache::Tstreqmap::Tstreqmap()
    : Text_noget_text_stat_question_ptrCHMap(bad_ptr)
{
}

Text_stat_cache::Tstreqmap::~Tstreqmap()
{
    for (Pix i = first(); i != 0; next(i))
    {
	delete contents(i);
    }
}
