// Fetch-on-demand map from local to global text numbers.
//
// $Id: text-mapping.h,v 1.7 1994/08/03 19:36:58 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef TEXT_MAPPING__
#define TEXT_MAPPING__

#include <DLList.h>
#include "kom-types.h"
#include "misc.h"
#include "conference.h"
#include "get_map_q.h"

class Local2Global;

class Text_mapping {
  public:
    Text_mapping();
    Text_mapping(Conference c, int chunk_size=50);
    Text_mapping(const Text_mapping&);
    ~Text_mapping();

    Text_mapping& operator=(const Text_mapping&);

    Bool valid() const;

    Text_no operator [](Local_text_no lno);
    // Prefetch FIRST. If it needs to be fetched from the server,
    // prefetch CHUNK_SIZE texts.
    Status prefetch(Local_text_no first, int chunk_size=50);
    void add(Local_text_no key, Text_no text);
    void add(const Local2Global& l2g);

    static void limit();

  private:

    Pix locate_interval(Local_text_no lno);
    void do_prefetch(Pix ix, Local_text_no lno, int wanted_chunk_size);
    void fix_results();

    class Interval {	
      public:
	virtual Text_no operator [](const Local_text_no lno) const =0;
	virtual Bool valid() const =0;
	virtual Bool joinable() const =0;
	virtual ~Interval();

	Local_text_no first;
	int	      chunk_size;
    };

    class Zero_interval : public Interval {
      public:
	Text_no operator [](const Local_text_no lno) const;
	Bool valid() const;
	Bool joinable() const;
    };

    class Unknown_interval : public Interval {
      public:
	Text_no operator [](const Local_text_no lno) const;
	Bool valid() const;
	Bool joinable() const;
	get_map_question *mq;
	~Unknown_interval();
    };

    class Known_interval : public Interval {
      public:
	Text_no operator [](const Local_text_no lno) const;
	Bool valid() const;
	Bool joinable() const;
	void join(const Known_interval &m);
	Text_no *array;
	~Known_interval();
    };

    class MCI {
      public:
	MCI(Conference c);
	MCI(const MCI&);
	~MCI();
	MCI& operator=(const MCI&);

	Conference conf;

	void add_ref();
	void sub_ref();

        // Extracted from the conf-stat.
	Local_text_no first_existing;
	Local_text_no first_non_existing;

	DLList<Interval*> maps;
	int refcount;
	int chunk_size;
    };
    
    MCI *mci;
};
#endif
