# The command interpreter for the nilkom client.
#
# $Id: nilkom_cmd.tcl,v 1.31 1994/08/10 07:24:27 inge Exp $
# Copyright (C) 1994  Per Cederqvist and Inge Wallin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


# ----------------------------------------------------------------
# global variables (local to this file)
#
#  nilkom_prompt - the current prompt. Determines the next command to use.
#                  Values beginnig with NK_ belong entirely to the TCL layer,
#                  other values originate from [kom_unread prompt].

# global variables (shared)
#  review_list   - list of textnos to review

# The command table. Each entry consists of 
#    - a user command template
#    - a TCL command template
#    - a pretty-printing user command name (optional)
#
# The user command template is a list of words.  The input the user
# enters must match the words in the user command template.  The
# template can contain argument placeholders.  The take one of the
# following forms:
#    %I   - An integer.
#    %R   - The rest of the line.
# In the future, more forms will be supported.  Example: %CONFUNIQ
# will mean a unique conference name.
#
# If the match routine can find a best-match from the table, the TCL
# command template will be used to form a TCL command to execute.  In
# its simplest form, a TCL command template is a TCL command to
# execute.  If the User command template contained a placeholder, the
# corresponding user-supplied argument is inserted when a word
# consisting of %<digit> is encountered.  %0 is the leftmost argument,
# %1 the next, and so on.
#
# The pretty-printing user command name is used by the help commands.

set cmdtab {
    {{vilka}
	{who_is_on_small}
	{vilka (r inloggade)}}
    {{lng vilkalista}
	{who_is_on_verbose}
	{(visa) lng vilkalista}}
    {{se tiden} 
	{se tiden}}
    {{sluta}
	{quit_command}}
    {{markera}
	{mark $senaste 127}
	{markera (senast lsta med markering 127)}}
    {{markera %I}
	{mark %0 127}
	{markera <inlggsnummer> (med markering 127)}}
    {{markera %I %I} 
	{mark %0 %1}
	{markera <inlggsnummer> (med markering) <markeringsnummer>}}
    {{avmarkera}
	{unmark $senaste}
	{avmarkera (senast lsta)}}
    {{avmarkera %I}
	{unmark %0}
	{avmarkera <inlggsnummer>}}
    {{terse senaste} 		
	{print_text $senaste}}
    {{terse %I} 		
	{print_text %0}
	{terse <inlggsnummer>}}
    {{terse text %I} 
	{print_text %0}
	{terse text <inlggsnummer>}}
    {{nsta mte} 	
	{next_conference}}
    {{g %R}
	{goto_named_conference %0}
	{g (till) <mtesnamn>}}
    {{kommentera} 
	{k}}
    {{kommentera %I} 
	{k %0}
	{kommentera <inlggsnummer>}}
    {{personligt}
	{start_personal_answer $senaste}
	{personligt (svar p senaste inlgget)}}
    {{personligt %I}
	{start_personal_answer %0}
	{personligt (svar p inlgg) <inlggsnummer>}}
    {{inlgg}
	{start_write_text}
	{(skriva) inlgg}}
    {{brev %R}
	{start_write_letter %0}
	{(skriva) brev (till) <namn>}}
    {{mottagare: %R}
	{create_text_add_recipient %0}
	{(lgg till) mottagare: <mtesnamn> (till det inlgg som editeras)}}
    {{extra kopia: %R}
	{create_text_add_cc_recipient %0}
	{(lgg till) extra kopia: <mtesnamn> (till det inlgg som editeras)}}
    {{addera kommentar: %I}
	{create_text_add_comment %0}
	{addera kommentar: <inlggsnummer> (till det inlgg som editeras)}}
    {{addera fotnot: %I}
	{create_text_add_footnote %0}
	{addera fotnot: <inlggsnummer> (till det inlgg som editeras)}}
    {{subtrahera %R}
	{subtract_recipient %0}
	{subtrahera <mtesnamn> (fungerar nnu bara p inlgg som editeras)}}
    {{fortstt}
	{create_text_resume_edit}
	{fortstt (editera inlgget)}}
    {{hela} 
	{create_text_redisplay; print "\n"}
	{(visa) hela (inlgget som editeras)}}
    {{snd till %R}
	{send_message_command %0}
	{snd (meddelande) till <personnamn>}}
    {{till %R}
	{send_message_command %0}
	{(snd meddelande) till <personnamn>}}
    {{snd alarmmeddelande}
	{send_alert_message_command}
	{snd alarmmeddelande (till alla)}}
    {{terse alla kommentarer}		
	{review_all_comments $senaste}}
    {{terse alla kommentarer %I}
	{review_all_comments %0}
	{terse alla kommentarer (till) <inlggsnumemr>}}
    {{terse kommenterade} 		
	{review_commented $senaste}
	{terse (det) kommenterade}}
    {{terse kommenteras av %I}
	{review_commented %0}
	{terse (det som) kommenteras av <inlggsnummer>}}
    {{terse presentation %R}
	{review_presentation %0}
	{terse presentation (fr) <mte/person>}}
    {{skicka} 
	{enter_text}
	{skicka (in texten)}}
    {{bort} 
	{throw_text}
	{(kasta) bort (inlgget som editeras)}}
    {{lsmarkera inte}
	{reject}
	{lsmarkera inte (senast lsta text)}}
    {{lista kommandon}
	{list_commands}
	{lista (alla) kommandon}}
    {{lista nyheter}
	{unread}
	{lista (alla) nyheter}}
    {{lista mten}
	{list_conferences}
	{lista (alla) mten}}
    {{lista personer}
	{list_persons}
	{lista (alla) personer}}
    {{medlem %R}
	{join_conference %0}
	{(bli) medlem (i) <mtesnamn>}}
    {{stt skrmstorlek default}
	{nilkom_page pagesize -1 -1}
	{stt skrmstorlek (till) default}}
    {{stt skrmstorlek ondlig}
	{nilkom_page pagesize 0 0}
	{stt skrmstorlek (till) ondlig}}
    {{stt skrmstorlek %I %I}
	{nilkom_page pagesize %0 %1}
	{stt skrmstorlek (till) <rader> <kolumner>}}
    {{hjlp}
	{help}}
    {{?}
	{help}
	{(Vad kan jag gra)?}}
    {{help}
	{help}}
}

set cmd_table [command_parser define $cmdtab]

proc list_commands {} {
    global cmdtab

    foreach cmd $cmdtab {
	if {[llength $cmd] > 2} {
	    set x [print "[lindex $cmd 2]\n"]
	} else {
	    set x [print "[lindex $cmd 0]\n"]
	}
	if {$x == "terminate"} {
	    return
	}
    }
}

proc print_prompt {} {
    global nilkom_prompt
    global review_list

    trim_review_list
    if {[llength $review_list] > 0} {
	set nilkom_prompt NK_REVIEW
    } else {
	set nilkom_prompt [kom_unread prompt]
    }

    nilkom_page sync

    case $nilkom_prompt in {
	NEXT_CONF     {print "G till nsta mte - "}
	NEXT_TEXT     {print "Lsa nsta text - "}
	NEXT_COMMENT  {print "Lsa nsta kommentar - "}
	NEXT_FOOTNOTE {print "Lsa nsta fotnot - "}
	NO_MORE_TEXT  {print "Se tiden - "}
	NK_REVIEW     {print "terse nsta - "}
    }
}

proc default_action {} {
    global nilkom_prompt
    global to_mark_as_read

    foreach tno $to_mark_as_read {
	kom_text mark_as_read $tno
    }
    set to_mark_as_read {}

    case $nilkom_prompt in {
      NEXT_CONF     {set new_conf [kom_unread goto_next_conf]; \
		     print [format "Du hamnade i %s (%d olsta)\n" \
				[kom_conference name $new_conf] \
				[lindex [kom_unread unread_texts_in_current] \
				        1]]}
      NEXT_TEXT     {read_text [kom_unread next_text remove]}
      NEXT_COMMENT  {read_text [kom_unread next_text remove]}
      NEXT_FOOTNOTE {read_text [kom_unread next_text remove]}
      NO_MORE_TEXT  {se tiden}
      NK_REVIEW     {review_next}
    }
}

proc isoify {str} {
    regsub -all \{ $str  str
    regsub -all \} $str  str
    regsub -all \\\| $str  str
    regsub -all \\\[ $str  str
    regsub -all \\\] $str  str
    regsub -all \\\\ $str  str
    return $str
}

# do_command is called whenever the user has entered a command.
# The command is present in $cmd.  If the command is empty, a default
# action is performed.  If the first character is "=", the remainder
# of the line is executed as TCL code.  Otherwise, the command table
# in $cmd_table is used to construct a TCL command to execute.  If no
# match is found, an error message is printed.
proc do_command {cmd} {
    global cmd_table
    global errorCode

    nilkom_page register "" "**MER** (SPC, RET, q)"
    if {[regexp {^ *$} $cmd]} {
	perform_command default_action
	return
    }

    if {[string range $cmd 0 0] == "="} {
	if {[catch {uplevel \#0 [string range $cmd 1 end]} res] == 0} {
	    if {[string length $res] > 0} {
	        print "$res\n"
	    }
	} else {
	    puts "Error: $res"
	    puts "errorCode: $errorCode"
	}
	return
    }

    # FIXME: This is bad for internationalization.
    set cmd [isoify $cmd]

    set match [command_parser match $cmd_table $cmd]

    if {[lindex $match 0]} {
	perform_command [lindex $match 1]
    } elseif {[lindex $match 1] == "NOMATCH"} {
	print "Jag frstr inte vad du menar\n"
    } else {
	report_ambiguous [lindex $match 2]
    }
}

proc perform_command {cmd} {
    global senaste
    global errorCode

    # puts "DOING-->$cmd<--"
    if {[catch {eval $cmd} res] != 0} {
	print "Error: $res"
	print "errorCode: $errorCode"
    }
}

proc report_ambiguous {alts} {
    print "Du kan mena ngot av fljande:\n"
    foreach m $alts {
        if {[print "  $m\n"] == "terminate"} {
	    return
	}	
    }
}

proc commandline {n} {
    global input_line
    global inhibit_prompt
    global quit_flag

    set inhibit_prompt 0

    switch $n {
	3
	    {return 1}
	10
	    -
	13  
	    {
		print "\n"
		do_command $input_line
	     	set input_line {}
		if {! $inhibit_prompt} {
	     	    print_prompt
		}
	    }
	8
	    -
	127
	    {
		set len [string length $input_line]
		if {$len == 0} {
		    nilkom_page beep
		} else {
		    nilkom_page remove \
			[string index $input_line [expr $len - 1]]
		    set input_line \
			[string range $input_line 0 [expr $len - 2]]
		}
	    }
	18
	    {print "\n$input_line"}
	21
	    {
		print "\n"
		set input_line {}
	    }
	default
	    {
		print [format "%c" $n]
		append input_line [format "%c" $n]
	    }
    }
    return $quit_flag
}
