// user-input.h - Input from the user
// 
// $Id: user-input.cc,v 1.1 1994/08/05 22:16:47 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <assert.h>
#include <tcl.h>
#include <stdlib.h>

#include "user-input.h"
#include "LyStr.h"
#include "connection.h"

// FIXME

User_input::User_input(IscMaster *master, int infd)
: in_master(master), pending_input(), idle_function(NULL)
{
  in_cb = isc_openfd(master, infd);
}

User_input::~User_input()
{
  isc_destroy(in_master, in_cb);
}

void
User_input::idle_callback(void (*callback)())
{
  idle_function = callback;
}

int
User_input::fill_pending_input(int timeout)
{
  IscEvent *ev = Connection::get_event(timeout);
  if (ev == &Connection::error_event)
    {
      return -1;
    }

  if (ev != NULL && ev->event != ISC_EVENT_TIMEOUT)
    {
      assert (ev->session == in_cb);
      switch(ev->event)
	{
	case ISC_EVENT_ERROR:
	  abort();
	case ISC_EVENT_MESSAGE:
	  pending_input << LyStr(ev->msg->buffer,
					 ev->msg->length);
	  break;
	default:
	  abort();
	}
    }
  else if (ev != NULL)
    {
      assert(ev->event == ISC_EVENT_TIMEOUT);
      if (timeout == 0)
	{
	  isc_dispose(ev);
	  return 0;
	}

      if (idle_function != NULL)
	(*idle_function)();
    }

  if (ev != NULL)
    isc_dispose(ev);

  return 0;
}

int
User_input::get_key()
{
  while (1)
    {
      if (pending_input.strlen() > 0)
	{
	  char res = pending_input[0];
	  pending_input.remove_first(1);
	  return res;
	}
      if (fill_pending_input(1000) < 0)
	return -1;
    }
}

int
User_input::pending()
{
  if (pending_input.strlen() == 0)
    fill_pending_input(0);
 
  return pending_input.strlen();
}


